#ifndef _QUANTIZELENGTH_CPP_
#define _QUANTIZELENGTH_CPP_

#include <iostream.h>

#include <reference.h>
#include <table.h>
#include <event.h>
#include <part.h>
#include <track.h>
#include <song.h>
#include <str.h>
#include <prMainEditor.h>

#include "quantizeLength.h"

extern Table * selectioN;
extern Table * selectionMemory;
extern PrMainEditor * mainEditor;

#define QUANTIZELENGTH_CONTEXT Addon::TREAT_SELECTION + Addon::TREAT_TRACK + Addon::TREAT_PART + Addon::TREAT_SONG

extern "C" {
  Addon * quantizeLength(Element * target) { return new QuantizeLength(target); }
  const char  * quantizeLength_name() { return "quantize length"; }
  const char  * quantizeLength_category() { return "quantize"; }
  int     quantizeLength_context() { return QUANTIZELENGTH_CONTEXT; }
}


QuantizeLength::QuantizeLength(Element * target) : Addon("quantize", target, TO_BE_UNDONE, QUANTIZELENGTH_CONTEXT), context(0), count(0) { run(); }

QuantizeLength::~QuantizeLength() {

}

void setLength(Event * ev, int grid) {
  long foo = ev->duration();
  foo += int(0.5*grid);
  foo -= foo%grid;
  if (foo==0) foo = grid;
  ev->setDuration(foo);
}

void QuantizeLength::selection(Table * selection) {
  context = Addon::TREAT_SELECTION;
  int grid = mainEditor->grid();
  int sz = selection->size();
  len = new long[sz];
  for (Reference * r = (Reference*)selection->first(); r != 0; r = (Reference*) Element::next(r)) {
    Event * ev = (Event*) r->getValue();
    len[count++] = ev->duration();
    setLength(ev,grid);
  }
}

void QuantizeLength::part(Part * part) {
  context = Addon::TREAT_PART;
  int grid = mainEditor->grid();
  int sz = part->size();
  len = new long[sz];
  for (Event * ev = (Event*)part->first(); ev != 0; ev = (Event*) Element::next(ev)) {
    len[count++] = ev->duration();
    setLength(ev,grid);
  }
}

void QuantizeLength::track(Track * track) {
  context = Addon::TREAT_TRACK;
  int grid = mainEditor->grid();
  int sz = track->size();
  int totalsz = 0;
  for (int i=0; i<sz; i++) {
    Part * pt = (Part*) track->get(i);
    totalsz += pt->size();
  }
  len = new long[totalsz];
  for (int i=0; i<sz; i++) {
    Part * pt = (Part*) track->get(i);
    for (Event * ev = (Event*) pt->first(); ev != 0; ev = (Event*) Element::next(ev)) {
      len[count++] = ev->duration();
      setLength(ev,grid);
    }
  }
}

void QuantizeLength::song(Song * song) {
  context = Addon::TREAT_SONG;
  int grid = mainEditor->grid();
  int totalsz = 0;
  int szs = song->size();
  for (int i=0; i<szs; i++) {
    Track * track = (Track*) song->get(i);
    int sz = track->size();
    for (int i=0; i<sz; i++) {
      Part * pt = (Part*) track->get(i);
      totalsz += pt->size();
    }
  }
  len = new long[totalsz];
  for (int i=0; i<szs; i++) {
    Track * track = (Track*) song->get(i);
    int sz = track->size();
    for (int i=0; i<sz; i++) {
      Part * pt = (Part*) track->get(i);
      for (Event * ev = (Event*) pt->first(); ev != 0; ev = (Event*) Element::next(ev)) {
	len[count++] = ev->duration();
	setLength(ev,grid);
      }
    }
  }
}

void QuantizeLength::undo() {
  long foo2 = 0;
  int mem = 0;
  if (context == Addon::TREAT_SELECTION) {
    Table * selection = (Table*) _target;
    for (Reference * r = (Reference*)selection->first(); r != 0; r = (Reference*) Element::next(r)) {
      Event * ev = (Event*) r->getValue();
      foo2 = ev->duration();
      ev->setDuration(len[mem]);
      len[mem++] = foo2;
    }
  } else if (context == Addon::TREAT_PART) {
    Part * part = (Part*) _target;
    for (Event * ev = (Event*)part->first(); ev != 0; ev = (Event*) Element::next(ev)) {
      foo2 = ev->duration();
      ev->setDuration(len[mem]);
      len[mem++] = foo2;
    }
  } else if (context == Addon::TREAT_TRACK) {
    Track * track = (Track*) _target;
    int sz = track->size();
    int totalsz = 0;
    for (int i=0; i<sz; i++) {
      Part * pt = (Part*) track->get(i);
      totalsz += pt->size();
    }
    for (int i=0; i<sz; i++) {
      Part * pt = (Part*) track->get(i);
      for (Event * ev = (Event*) pt->first(); ev != 0; ev = (Event*) Element::next(ev)) {
	foo2 = ev->duration();
	ev->setDuration(len[mem]);
	len[mem++] = foo2;
      }
    }
  } else if (context == Addon::TREAT_SONG) {
    Song * song = (Song*) _target;
    int totalsz = 0;
    int szs = song->size();
    for (int i=0; i<szs; i++) {
      Track * track = (Track*) song->get(i);
      int sz = track->size();
      for (int i=0; i<sz; i++) {
	Part * pt = (Part*) track->get(i);
	totalsz += pt->size();
      }
    }
    for (int i=0; i<szs; i++) {
      Track * track = (Track*) song->get(i);
      int sz = track->size();
      for (int i=0; i<sz; i++) {
	Part * pt = (Part*) track->get(i);
	for (Event * ev = (Event*) pt->first(); ev != 0; ev = (Event*) Element::next(ev)) {
	  foo2 = ev->duration();
	  ev->setDuration(len[mem]);
	  len[mem++] = foo2;
	}
      }
    }
  }
}

void QuantizeLength::redo() {
  long foo2 = 0;
  int mem = 0;
  if (context == Addon::TREAT_SELECTION) {
    Table * selection = (Table*) _target;
    for (Reference * r = (Reference*)selection->first(); r != 0; r = (Reference*) Element::next(r)) {
      Event * ev = (Event*) r->getValue();
      foo2 = ev->duration();
      ev->setDuration(len[mem]);
      len[mem++] = foo2;
    }
  } else if (context == Addon::TREAT_PART) {
    Part * part = (Part*) _target;
    for (Event * ev = (Event*)part->first(); ev != 0; ev = (Event*) Element::next(ev)) {
      foo2 = ev->duration();
      ev->setDuration(len[mem]);
      len[mem++] = foo2;
    }
  } else if (context == Addon::TREAT_TRACK) {
    Track * track = (Track*) _target;
    int sz = track->size();
    int totalsz = 0;
    for (int i=0; i<sz; i++) {
      Part * pt = (Part*) track->get(i);
      totalsz += pt->size();
    }
    for (int i=0; i<sz; i++) {
      Part * pt = (Part*) track->get(i);
      for (Event * ev = (Event*) pt->first(); ev != 0; ev = (Event*) Element::next(ev)) {
	foo2 = ev->duration();
	ev->setDuration(len[mem]);
	len[mem++] = foo2;
      }
    }
  } else if (context == Addon::TREAT_SONG) {
    Song * song = (Song*) _target;
    int totalsz = 0;
    int szs = song->size();
    for (int i=0; i<szs; i++) {
      Track * track = (Track*) song->get(i);
      int sz = track->size();
      for (int i=0; i<sz; i++) {
	Part * pt = (Part*) track->get(i);
	totalsz += pt->size();
      }
    }
    for (int i=0; i<szs; i++) {
      Track * track = (Track*) song->get(i);
      int sz = track->size();
      for (int i=0; i<sz; i++) {
	Part * pt = (Part*) track->get(i);
	for (Event * ev = (Event*) pt->first(); ev != 0; ev = (Event*) Element::next(ev)) {
	  foo2 = ev->duration();
	  ev->setDuration(len[mem]);
	  len[mem++] = foo2;
	}
      }
    }
  }
}






#endif
