#ifndef _ALSAPLAYER_CPP_
#define _ALSAPLAYER_CPP_

#include <iostream>
#include <string>

using namespace std;

#include "config.h"

#include "alsaPlayer.h"
#include "reference.h"
#include "part.h"
#include "track.h"
#include "note.h"
#include "midiEvent.h"
#include "masterEvent.h"
#include "audioEvent.h"
#include "song.h"
#include "table.h"
#include "element.h"
#include "scoreTrack.h"
#include "drumTrack.h"
#include "prProgress.h"
#include "prMainEditor.h"
#include "songIterator.h"
#include "addElement.h"

#include "alsaManager.h"

extern Table * selectioN;

extern PrMainEditor * mainEditor;


AlsaPlayer::AlsaPlayer() : Player() {
  init();
}

AlsaPlayer::AlsaPlayer(Song * s) : Player(s) {
  init();
}

AlsaPlayer::~AlsaPlayer() {
  freePlayer();
}



void AlsaPlayer::init() {
  _player_name = strdup("Alsa");
  initPlayer();
}

const char * AlsaPlayer::device(int i) {
  const char * dev = 0;
#ifdef HAVE_ALSA
  dev = alsa->device(i);
#endif
  return dev;
}

int AlsaPlayer::devicenum() {
  int ret = 0;
#ifdef HAVE_ALSA
  ret = alsa->devnum();
#endif
  return ret;
}

void AlsaPlayer::play(PrProgress * progress, Part * part, long left, long right, bool cycl, int record_track) {
  playStart(progress, part, left, right);
  mainEditor->startTimer();
}


void AlsaPlayer::hit(int out, int ch, int freq, int vol) {
  if (!_playing) {
    _playing = true;
#ifdef HAVE_ALSA
    alsa->hit(out, ch, freq, vol);
#endif
    _playing = false;
  }
}

void AlsaPlayer::quad(int out, int ch, int freq1, int freq2, int freq3, int freq4, int vol) {
  if (!_playing) {
    _playing = true;
    int q[4] = { freq1, freq2, freq3, freq4 };
#ifdef HAVE_ALSA
    alsa->hit(out, ch, 4, q, vol);
#endif
    _playing = false;
  }
}


double AlsaPlayer::time() {
#ifdef HAVE_ALSA
  return alsa->time();
#endif
  return 0.0;
}
  
long AlsaPlayer::ticks() {
#ifdef HAVE_ALSA
  return alsa->ticks();
#endif
  return 0L;
}

long AlsaPlayer::volume() {
  return 0; //ps->volume;
}

void AlsaPlayer::initPlayer() {
#ifdef HAVE_ALSA
  alsa = new AlsaManager();
  if (_song != 0) setTempo(_song->tempo());
#endif
}

void AlsaPlayer::freePlayer()
{
  // TODO!
}

bool AlsaPlayer::playStart(PrProgress * progress, Part * part, long lft, long rgt) {
  if (!_playing) {
    _playing = true;

#ifdef HAVE_ALSA
    ntracks = _song->size();
    setTempo(_song->tempo());
    aiterator = new SongIterator(_song, lft, rgt);
    for (int t=0; t<ntracks; t++) {
      if (aiterator->channel(t)!=-1) alsa->prgchange(0, aiterator->channel(t), aiterator->program(t));
    }
    pp = lft;
    left = pp;
    alsa->playStart(left);
#endif
  }
  return true;
}

bool AlsaPlayer::playQuantum(bool cycl) {
  bool ret = true;
#ifdef HAVE_ALSA
    Event * event = 0;
    Note * note   = 0;
    Track * tr    = 0;
    ::MidiEvent * mev;
    int temp = 0;
    while ((pp < alsa->ticks() + 768) && !aiterator->done()) {
      for (int t=0; t<ntracks; t++) {
	tr = aiterator->track(t);
	while (aiterator->startsAt(t,pp)) {
	  if (!aiterator->mute(t)) {
	    // new program settings?
	    if (aiterator->change(t)!=0) alsa->prgchange(pp, aiterator->channel(t), aiterator->program(t));
	    event = (Event*) (*aiterator)[t];
	    if (event->isA()==NOTE) {
	      note = (Note*) event;
	      alsa->playNote(0, aiterator->channel(t), tr->pitch(note), tr->vel(note), pp + tr->delay(), note->duration());
	    } else if (event->isA()==MASTEREVENT) {
	      temp = ((MasterEvent*) event)->tempo();
	      if (temp>0 && ( tr->isA()==MASTERTRACK || _song->master()==0 )) {
		setTempo(temp, pp);
		_song->setTempo(temp);
	      } else {
		_song->setMeter0(((MasterEvent*) event)->meter0());
		_song->setMeter1(((MasterEvent*) event)->meter1());
	      }
	    } else if (event->isA()==MIDIEVENT) {
	      mev = (::MidiEvent*) event;
	      alsa->event(mev->code(), mev->channel(), mev->value1(), mev->value2(), pp + tr->delay());
	
	    } else if (event->isA()==AUDIOEVENT) {
	      cout << "play " << ((AudioEvent*)event)->fileName() << endl;
	    } else {
	      cout << "track " << t << *event << endl;
	    }
	  }
	  aiterator->increment(t);
	}
      }
      pp++;
    }
    ret = !(aiterator->done() && pp>=alsa->ticks());
#endif
    return ret;
}

void AlsaPlayer::killAllNotes() {
#ifdef HAVE_ALSA
  alsa->playEnd();
#endif
  _playing = false;
}

void AlsaPlayer::setTempo(int t, long pos) {
#ifdef HAVE_ALSA
  alsa->tempo(t, pos);
#endif
}

void AlsaPlayer::setPosition(Position p) {
#ifdef HAVE_ALSA
  alsa->playEnd();
  alsa->alloff();
  mainEditor->stopTimer();
  _playing = false;
  pp = p.ticks();
  playStart(0,0,pp,0);
  mainEditor->startTimer();
#endif
}

void AlsaPlayer::mergeQueue() {
  // TODO
}



void AlsaPlayer::programChange(Track * tr) {
  int ch  = tr->channel();
  int prg = tr->program();
#ifdef HAVE_ALSA
  alsa->prgchange(0, ch, prg);
#endif
}


void AlsaPlayer::metronome(int p) {
  // TODO!
}

// vim:ts=8:sw=2:sts=2

#endif
