#ifndef _KDEMASTERCONTENT_CPP_
#define _KDEMASTERCONTENT_CPP_

#include <stdio.h>
#include <qpainter.h>
#include <kiconloader.h>
#include <klocale.h>

#include "kdeMasterEditor.h"
#include "kdeMasterContent.h"
#include "kdeEditorToolBar.h"
#include "kdeMainEditor.h"
#include "positionCursor.h"
#include "masterEvent.h"
#include "part.h"

extern PrMainEditor * mainEditor;

#define ME ((KdeMainEditor*)mainEditor)


KdeMasterContent::KdeMasterContent(QWidget * parent, KdeMasterEditor * ed , KdeEditorToolBar * tb)
  : QFrame(parent,"content",QFrame::Sunken), _editor(ed), _toolbar(tb), _xoffset(24)
{
  
  KIconLoader * loader = KGlobal::iconLoader();
  if (ME->editorBackground() == ::NoBackground) {
    setBackgroundColor( Qt::white );
    setBackgroundMode( QWidget::FixedColor );
  } else if (ME->editorBackground() == PaperBackground) {
    QPixmap paper = loader->loadIcon( "paper02", KIcon::Toolbar );
    setBackgroundPixmap(paper);
    setBackgroundMode( QWidget::FixedPixmap );
  } else if (ME->editorBackground() == UserDefinedBackground) {
    // QPixmap paper = loader->loadIcon( ME->userBackground(), KIcon::Toolbar );
  }
  _position_cursor = new PositionCursor(this,_xoffset);
}

void KdeMasterContent::paintEvent(QPaintEvent*) {
  QPixmap pix( width(), height() );
  pix.fill( this, 10,10 );
  QPainter painter;
  painter.begin( &pix );

  Position _leftPos = _editor->left();
  Position _rightPos = _editor->right();

  int hh = height();
  int ww = width();
  int y0 = hh-_xoffset;
  int x0 = _xoffset;
  double ppt = (ww-x0)*1.0/(_rightPos-_leftPos.ticks());

  painter.drawRect(0, 0, ww, hh);

  painter.drawLine(x0-2, y0, ww, y0);
  painter.drawLine(x0, 12, x0, y0+2);

  Part * pt = _editor->part();
  Event * ev;
  MasterEvent * me;
  long pos;
  int tempo;
  char * legend = new char[30];
  for (Position x = _leftPos; x < _rightPos; x.nextBeat()) {
    int xx = int((x-_leftPos.ticks())*ppt)+x0;
    painter.drawLine(xx, y0+2, xx, y0-2);
    if (x.beat()==1) {
      painter.drawLine(xx, y0+4, xx, y0-4);
      sprintf(legend, "%d", x.bar());
      painter.drawText(xx, y0+16, legend);
    }
  }
  
  int xx1 = x0;
  int xx0 = x0;
  int yy0 = y0;
  int yy1 = y0;
  for (Iterator it = Iterator(pt, 0, _rightPos); !it.done(); it++) {
    ev = (Event*) (*it);
    if (ev->isA()==MASTEREVENT) {
      me = (MasterEvent*) ev;
      pos = (pt->start(me)-_leftPos).ticks();
      tempo = me->tempo();
      if (tempo>0) {
	xx0 = pos*ppt+x0;
	yy0  = y0-tempo*0.4;
	if (xx0 >= _xoffset) {
	  if (xx1 < _xoffset) xx1 = _xoffset;
	  painter.drawLine(xx1, yy1, xx0, yy1);
	  painter.drawLine(xx0, yy1, xx0, yy0);
	}
	xx1 = xx0;
	yy1 = yy0;
      }
    }
  }
  if (xx1 < _xoffset) xx1 = _xoffset;
  painter.drawLine(xx1, yy1, ww, yy1);
  
  delete legend;
  painter.end();
  bitBlt( this, 0, 0, &pix );

}

void KdeMasterContent::mousePressEvent ( QMouseEvent * ) {

}

void KdeMasterContent::mouseMoveEvent  ( QMouseEvent * ) {

}

void KdeMasterContent::mouseReleaseEvent ( QMouseEvent * ) {

}

void KdeMasterContent::keyPressEvent ( QKeyEvent * key ) {

  switch( key->key() ) {
  default:
    key->ignore();
  }

}

void KdeMasterContent::keyReleaseEvent ( QKeyEvent * key ) {

  switch( key->key() ) {
  default:
    key->ignore();
  }

}

void KdeMasterContent::moveCursor(int ticks) {
  double ppt = (width()-_xoffset)*1.0/(_editor->right()-_editor->left().ticks());
  long pos = long((ticks-_editor->left().ticks())*ppt);
  if (pos>width()-24) {
    _editor->moveleft(_editor->numberOfBars());
  }
  _position_cursor->move(pos);

}


#endif
