/***************************************************************************
                          pattern.h  -  description
                             -------------------
    begin                : Sat Jun 8 2002
    copyright            : (C) 2002 by Morten Hustveit
    email                : mortehu@ifi.uio.no
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef PATTERN_H
#define PATTERN_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/**
  *@author Morten Hustveit
  */

// include files for Qt
#include <qwidget.h>
#include <qvaluelist.h>

#include "sanetrackerdoc.h"

class QLabel;
class QHBoxLayout;
class QVBoxLayout;
class QComboBox;
class QScrollBar;
class QPushButton;
class QSpacerItem;
class QCheckBox;
class QSpinBox;
class KPopupMenu;
class Configuration;
class Scheduler;

class Pattern : public QWidget
{
Q_OBJECT

public:

  Pattern(QWidget* parent, SanetrackerDoc& doc, const Configuration*, Scheduler&);
  ~Pattern();

  bool active() const { return _active; }
  void setActive(bool _active = true);

signals:

  void activated();
  void nextPattern();
  void prevPattern();

public slots:

  void keyPressed(QKeyEvent*);
  void keyPressed(uint note, uint velocity);
  void keyReleased(uint note, uint velocity);
  void edit(uint device, uint pattern, uint position);

protected slots:

  void clearTrack();
  void copyTrack();
  void pasteTrack();
  void transposeTrackOctaveUp();
  void transposeTrackOctaveDown();
  void clearPattern();
  void copyPattern();
  void pastePattern();
  void transposePatternOctaveUp();
  void transposePatternOctaveDown();

  void patchActivated(int);

private:

  bool _active;

  friend class Data : public QWidget
  {
  public:

    Data(Pattern* parent);

  protected:

    friend class Pattern;

    Pattern* parent;

    void scroll(int rows);
    void updateCell(uint row, uint col);

    void paintEvent(QPaintEvent* event);
    void mousePressEvent(QMouseEvent* event);
    void mouseReleaseEvent(QMouseEvent* event);
    void mouseMoveEvent(QMouseEvent* event);
    void wheelEvent(QWheelEvent*);
  };

  SanetrackerDoc& doc;
  const Configuration* config;
  Scheduler& scheduler;

  KPopupMenu*  menu;
  QHBoxLayout* hLayout;
  QVBoxLayout* vLayout;
  QLabel*      labelPatch;
  QComboBox*   comboPatches;
  Data*        data;
  QScrollBar*  scrollHoriz;
  QLabel*      labelStep;
  QSpinBox*    spinStep;
  QCheckBox*   checkContinuous;

  uint device;
  uint pattern;
  uint position;
  uint row;
  uint col;
  uint selectRow;
  uint selectCol;
  uint subFocus;
  uint subFocusMax;
  QValueList<uint> activeNotes;

  enum TextMode
  {
    Background = 0,
    BackgroundDark = 1,
    Highlight = 2
  };

  static const char* font[25][9];
  static QPixmap* fontPixmaps[3][25];
  static void drawText(QPainter&, int x, int y, const QString& string, TextMode mode = Background);
};

#endif

